// jARToolKit Version 2.0
import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.geometry.Sphere;
import java.awt.BorderLayout;
import java.awt.GraphicsConfiguration;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Bounds;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.Locale;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.VirtualUniverse;

import javax.swing.JFrame;
import javax.swing.JPopupMenu;

import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3f;

import net.sourceforge.jartoolkit.java3d.Behavior.ARBehavior;
import net.sourceforge.jartoolkit.java3d.util.JARToolKit3D;
import com.sun.j3d.utils.universe.SimpleUniverse;
import java.util.Timer;
import java.util.TimerTask;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.Transform3D;



public class JARToolKit3D_Demo extends JFrame implements KeyListener {
    private Bounds bounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0), Double.MAX_VALUE);
    
    // Drawing canvas for 3D rendering
    private Canvas3D canvas;
    
    // BranchGroup for the Content Branch of the scene
    private BranchGroup contentBranch;
    
    // TransformGroup  node of the scene contents
    private TransformGroup contentsTransGr;
    
    // Locale of the scene graph.
    private Locale locale;
    private ARBehavior m_arBehavior = null;
    private JARToolKit3D m_JARToolKit3D = null;
    private TransformGroup objTrans;
    private TransformGroup objTrans2;
    private TransformGroup sphereTG;
    private TransformGroup coneTG;
    private TransformGroup coneTopTG;
    // Virtual Universe object.
    private VirtualUniverse universe;
    private Timer t;
    private float radius = 30f;
    private PolygonAttributes pa = new PolygonAttributes();
    
    public static void main(String[] args) {
        JARToolKit3D_Demo myframe = new JARToolKit3D_Demo();
        
        myframe.setVisible(true);
    }
    
    public JARToolKit3D_Demo() {
        super("Java3D Example using JARToolKit3D");
        
        // Creating and setting the Canvas3D
        GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
        
        canvas = new Canvas3D(config);
        pa.setCapability(pa.ALLOW_MODE_READ);
        pa.setCapability(pa.ALLOW_MODE_WRITE);        
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(canvas, "Center");
        
        // Setting the VirtualUniverse and the Locale nodes
        setUniverse();
        
        try {
            m_JARToolKit3D = JARToolKit3D.create();
            
            m_JARToolKit3D.initialize(JARToolKit3D.getAbsolutePath("data/camera_para.dat"));
            
            // Create the viewing branch
            locale.addBranchGraph(m_JARToolKit3D.createViewingBranch(canvas));
        } catch(Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        
        // Setting the content branch
        setContent();
        
        // To avoid problems between Java3D and Swing
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        
        // enabling window closing
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        canvas.addKeyListener(this);
        setSize(m_JARToolKit3D.getWidth(), m_JARToolKit3D.getHeight());

              setResizable(false);
        t = new Timer();
        t.scheduleAtFixedRate(new TrackingTask(), 50, 50);
    }
    
    private void setTGCapabilities(TransformGroup tg) {
        tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    }
    
    public void keyPressed(KeyEvent e) {
    }
    
    public void keyReleased(KeyEvent e) {
        switch(e.getKeyCode()) {
            case KeyEvent.VK_UP :
                m_arBehavior.setThreshold(m_arBehavior.getThreshold() + 10);
                System.out.println("threshold = " + m_arBehavior.getThreshold());
                break;
                
            case KeyEvent.VK_DOWN :
                m_arBehavior.setThreshold(m_arBehavior.getThreshold() - 10);
                System.out.println("threshold = " + m_arBehavior.getThreshold());
                break;
                
            default :
                return;
        }
    }
    
    public void keyTyped(KeyEvent e) {
    }
    
    private void setContent() {
        // Creating the content branch
        contentsTransGr = new TransformGroup();
        
        contentsTransGr.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        setLighting();
        
        try {
            contentsTransGr.addChild(m_JARToolKit3D.createBackground());
            
            m_arBehavior = m_JARToolKit3D.createRecognition();
            
            contentsTransGr.addChild(m_arBehavior);
            
            objTrans = m_JARToolKit3D.createPatternTransform(JARToolKit3D.getAbsolutePath("data/pattern/hiroPatt"), 60, 0, 5f, false);
            objTrans2 = m_JARToolKit3D.createPatternTransform(JARToolKit3D.getAbsolutePath("data/pattern/patt.kanji"), 60, 0, 5f, false);
            
            m_arBehavior.registerObject(objTrans);
            m_arBehavior.registerObject(objTrans2);
        } catch(Exception e) {
            System.err.println(e);
            System.exit(0);
        }
        
        contentsTransGr.addChild(objTrans);
        contentsTransGr.addChild(objTrans2);
        
        
        
        //ColorCube cyl1 = new ColorCube(20.0f);
        Cone cone1 = new Cone(15f, 50f);
        
        Appearance appearance = new Appearance();
        
        cone1.setAppearance(appearance);
        
        Transform3D coneTrans = new Transform3D();
        coneTrans.setTranslation(new Vector3f(0, 25f, 0));
        coneTG = new TransformGroup(coneTrans);
        
        
        objTrans.addChild(coneTG);
        coneTG.addChild(cone1);
        
        coneTopTG = new TransformGroup();
        
        Transform3D coneTopTrans = new Transform3D();
        
        coneTopTrans.setTranslation(new Vector3f(0,  50f,0));
        coneTopTG.setTransform(coneTopTrans);
        this.setTGCapabilities(coneTopTG);
        
        Sphere coneTop = new Sphere(2f);
        
        coneTopTG.addChild(coneTop);
        objTrans.addChild(coneTopTG);
        
        
        Sphere sphere1 = new Sphere(radius);
        Appearance appearance2 = new Appearance();
        appearance2.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);
        appearance2.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
        appearance2.setPolygonAttributes(pa);
        sphere1.setAppearance(appearance2);
        Transform3D sphereTrans = new Transform3D();
        sphereTrans.setTranslation(new Vector3f(0, 15f, 0));
        sphereTG = new TransformGroup(sphereTrans);
        
        objTrans2.addChild(sphereTG);
        sphereTG.addChild(sphere1);
        
        this.setTGCapabilities(sphereTG);
        this.setTGCapabilities(coneTG);
        this.setTGCapabilities(objTrans);
        this.setTGCapabilities(objTrans2);
        
        contentBranch = new BranchGroup();
        
        contentBranch.addChild(contentsTransGr);
        
        // Compiling the branch graph before making it live
        contentBranch.compile();
        
        // Adding a branch graph into a locale makes its nodes live (drawable)
        locale.addBranchGraph(contentBranch);
    }
        
    private void setLighting() {
        AmbientLight ambientLight = new AmbientLight();
        
        ambientLight.setEnable(true);
        ambientLight.setColor(new Color3f(0.10f, 0.1f, 1.0f));
        ambientLight.setCapability(AmbientLight.ALLOW_STATE_READ);
        ambientLight.setCapability(AmbientLight.ALLOW_STATE_WRITE);
        ambientLight.setInfluencingBounds(bounds);
        contentsTransGr.addChild(ambientLight);
        
        DirectionalLight dirLight = new DirectionalLight();
        
        dirLight.setEnable(true);
        dirLight.setColor(new Color3f(1.0f, 0.0f, 0.0f));
        dirLight.setDirection(new Vector3f(1.0f, -0.5f, -0.5f));
        dirLight.setCapability(AmbientLight.ALLOW_STATE_WRITE);
        dirLight.setInfluencingBounds(bounds);
        contentsTransGr.addChild(dirLight);
    }
    
    private void setUniverse() {
        // Creating the VirtualUniverse and the Locale nodes
        universe = new VirtualUniverse();
        locale = new Locale(universe);
    }
    
    protected class TrackingTask extends TimerTask {
        public void run() {
            Transform3D inverse_ConeMarker_Top = new Transform3D();
            //Transform3D inverse_ConeMarker_Cone = new Transform3D();
            Transform3D inverse_Root_ConeMarker = new Transform3D();
            Transform3D root_SphereMarker = new Transform3D();
            Transform3D sphereMarker_Sphere = new Transform3D();
            
            coneTopTG.getTransform(inverse_ConeMarker_Top);
            inverse_ConeMarker_Top.invert();
            objTrans.getTransform(inverse_Root_ConeMarker);
            inverse_Root_ConeMarker.invert();
            objTrans2.getTransform(root_SphereMarker);
            sphereTG.getTransform(sphereMarker_Sphere);
            
            Transform3D temp1 = new Transform3D();
            temp1.mul(inverse_ConeMarker_Top, inverse_Root_ConeMarker);
            
            Transform3D temp2 = new Transform3D();
            temp2.mul(temp1, root_SphereMarker);
            
            Transform3D temp3 = new Transform3D();
            temp3.mul(temp2, sphereMarker_Sphere);
            
            Vector3f vector = new Vector3f();
            temp3.get(vector);
            if (vector.length()<radius) {
                pa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
            } else {
                pa.setPolygonMode(PolygonAttributes.POLYGON_FILL);
            }
            
            
        }
    }
}
